package	jp.co.sds_corp.util.daemon.filter;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.core.Schedule;

/**
* 時間の範囲を定義します。
* TimerDaemon の起動可能な時間を定義するために使用されます。
<P>
XMLは以下の形式です。
<P>
許可する時間を指定します。
<PRE>
&lt;Filter class="jp.co.sds_corp.util.daemon.filter.TimeFilter"
	type="allow" from="06:00:00" to="21:00:00"/&gt;
</PRE>
拒否する時間を指定します。
<PRE>
&lt;Filter class="jp.co.sds_corp.util.daemon.filter.TimeFilter"
	type="deny" from="06:00:00" to="21:00:00"/&gt;
</PRE>
{@link Schedule}形式の設定を使用することも可能です。
ただしFrom要素とTo要素のサイクルがあっていなくてはなりません。<BR>
<PRE>
&lt;Filter class="jp.co.sds_corp.util.daemon.filter.TimeFilter"
	type="deny"&gt;
	&lt;From&gt;
		&lt;dayofweek&gt;金&lt;/dayofweek&gt;
		&lt;time&gt;18:00:00&lt;/time&gt;
	&lt;/From&gt;
	&lt;To&gt;
		&lt;dayofweek&gt;金&lt;/dayofweek&gt;
		&lt;time&gt;18:00:00&lt;/time&gt;
	&lt;/To&gt;
&lt;/Filter&gt;
</PRE>
以前のバージョンとの互換性のため、以下の形式も一時的に使用可能ですが
将来のバージョンでは削除されます。
<PRE>
&lt;TimeRange type="deny" from="06:00:00" to="21:00:00"/&gt;
</PRE>
*/
public class TimeFilter extends AbstractFilter
{
	private Schedule	fr;
	private Schedule	to;

	/** 
	* 時間帯を初期化します。
	* @param	config	設定情報。<TimeRange>を処理します。
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		IConfiguration attr = config.getAttributes();
		String var;

		var = attr.get( "from", null );
		if ( var != null ) {
			fr = new Schedule();
			fr.setTime( var );
			var = attr.get( "to" );
			to = new Schedule();
			to.setTime( var );
		} else {
			IConfiguration c;
			c = config.getChild( "From" );
			fr = new Schedule();
			fr.init( c );
			c = config.getChild( "To" );
			to = new Schedule();
			to.init( c );

			if ( fr.getCycle() != to.getCycle() ) {
				throw( new IllegalArgumentException(
							"FromとToが不一致です。" ));
			}
		}
	}

	/**
	* 現在時刻がこの時間帯に含まれるかどうかをチェックします。
	* @return	この時間帯に含まれるか否かのチェック結果
	*/
	public boolean isValid() {
		CmnDate dt = new CmnDate();
		if ( fr.compareTo( to ) < 0 ) {
			if ( fr.compareTo( dt ) <= 0 && to.compareTo( dt ) >= 0 ) {
				return true;
			}
		} else {
			if ( fr.compareTo( dt ) <= 0 || to.compareTo( dt ) >= 0 ) {
				return true;
			}
		}
		return false;
	}

	/**
	* この時間帯の文字列表現。
	* @return	文字列表現。
	*/
	public String toString() {
		String ret = super.toString() + ":" + fr + "-" + to;
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param    config	設定情報
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		c = config.addChild( "From" );
		fr.saveConfig( c );
		c = config.addChild( "To" );
		to.saveConfig( c );
	}
}

