package	jp.co.sds_corp.util.daemon.manager;

import java.util.Date;
import java.util.Iterator;

import javax.management.AttributeChangeNotification;
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.NotificationBroadcasterSupport;
import javax.management.ObjectName;

import com.sun.jdmk.comm.HtmlAdaptorServer;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.IService;
import jp.co.sds_corp.util.daemon.NestedService;
import jp.co.sds_corp.util.daemon.core.IDaemon;
import jp.co.sds_corp.util.daemon.core.IStatusChangeListener;
import jp.co.sds_corp.util.daemon.core.StatusChangeEvent;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* デーモンマネージャです。
* 指定されたXMLファイルから、デーモンの情報を取得してインスタンスを生成します。
* また、JMXのI/Fを持ち、遠隔オペレーションやデーモンの状態表示等を
* サポートします。
<P>
XMLファイルは以下のような形式です。
<PRE>
&lt;?xml version="1.0" encoding="UTF-8"?&gt;
&lt;Domain&gt;
</PRE>

&lt;MBean&gt;要素には、起動するMBeanの情報を記述します。
<PRE>
&lt;MBean name="Connector:name=RMIConnector"
	class="jp.co.sds_corp.util.jmx.rmi.server.RMIConnectorService"&gt;
&lt;/MBean&gt;
</PRE>
MBean要素以外は{@link DaemonManager}を参照してください。
*/
public class JMXManager extends DaemonManager
		implements IStatusChangeListener, JMXManagerMBean
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	MBeanServer	server;
	NotificationBroadcasterSupport broadcaster;

	/**
	* デーモンマネージャを生成します。
	*/
	protected JMXManager() {
		logger.debug( this, "start:" + getClass().getName() );
	}

	/**
	* デーモンマネージャのインスタンスを返します。
	*/
	public static DaemonManager getInstance() {
		if ( instance == null ) {
			instance = new JMXManager();
			Thread shut = new Thread() {
				public void run() {
					instance.onShutdown();
				}
			};
			Runtime.getRuntime().addShutdownHook( shut );
		}
		return instance;
	}

	/**
	* デーモンマネージャを、指定されたXMLファイルで初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config )
		throws Exception
	{
		server = MBeanServerFactory.createMBeanServer();
		broadcaster = new NotificationBroadcasterSupport();
		setupJMX( server, config );

		super.init( config );
	}

	/**
	* JMXを初期化します。
	* @param	server	JMXのMBeanServer
	* @param	config	設定情報
	*/
	protected void setupJMX( MBeanServer server, IConfiguration config )
		throws Exception
	{
		createHtmlAdaptor( server, config );
		createMBean( server, config );

		ObjectName objName = null;
		objName = new ObjectName( "DaemonManager:name=manager" );
		server.registerMBean( this, objName );
	}

	/**
	* ManagedBean を生成します。
	* @param	server	JMXのMBeanServer
	* @param	config	設定情報
	*/
	protected void createMBean( MBeanServer server, IConfiguration config)
		throws Exception
	{
		Iterator itr = null;
		try {
			itr = config.getChilds( "MBean" );
		} catch ( Exception ex ) {
		}
		if ( itr == null ) {
			return;
		}

		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			String name = c.getAttributes().get( "name" );
			String cls = c.getAttributes().get( "class" );
			ObjectName objName = null;
			objName = new ObjectName( name );
			server.createMBean( cls, objName );
			server.invoke( objName, "init",
				new Object[] { c },
				new String[] { "jp.co.sds_corp.util.config.IConfiguration" } );
			//server.invoke( objName, "init", new Object[] {}, new String[] {} );
			server.invoke( objName, "start", new Object[] {}, new String[] {} );
			logger.debug( this, name + "started." );
		}
	}

	/**
	* HtmpAdaptorを生成します。
	* @param	server	JMXのMBeanServer
	* @param	config	設定情報
	*/
	protected void createHtmlAdaptor( MBeanServer server, IConfiguration config)
		throws Exception
	{
		IConfiguration c = null;
		Integer port = null;
		try {
			c = config.getChild( "HtmlAdaptor" );
			port = Integer.valueOf( c.get( "port" ));
		} catch ( Exception ex ) {
		}
		if ( port == null ) {
			return;
		}
		HtmlAdaptorServer hadaptor = new HtmlAdaptorServer();
		ObjectName objName = null;
		objName = new ObjectName( "Adaptor:name=hadaptor,port=" + port );
		server.registerMBean( hadaptor, objName );
		hadaptor.start();
		logger.debug( this, "HtmlAdaptorService started." );
	}

	/**
	* 指定されたデーモンを生成します。
	* @param	config	設定情報
	* @param	i		duplicate
	*/
	public IDaemon createDaemon( IConfiguration config, int i )
		throws Exception
	{
		IDaemon dmn = super.createDaemon( config, i );
		addDaemon( dmn );
		IService svc = dmn.getService();
		addService( svc );
		if ( svc instanceof NestedService ) {
			Iterator itr = ((NestedService )svc).getChildServices();
			while ( itr.hasNext()) {
				IService cld = (IService )itr.next();
				addService( cld );
			}
		}
		return dmn;
	}

	/**
	* デーモンをMBeanServerに登録します。
	*/
	protected void addDaemon( IDaemon dmn ) throws Exception {
		ObjectName objName = null;
		objName = new ObjectName( "Daemon:name=" + dmn.getName() );
		server.registerMBean( dmn, objName );
		logger.debug( this, "add daemon:" + dmn.getName()
								+ ":" + dmn.getClass().getName() );
	}

	/**
	* サービスをMBeanServerに登録します。
	*/
	protected void addService( IService svc ) throws Exception {
		try {
			ObjectName objName = null;
			objName = new ObjectName( "Service:name=" + svc.getName() );
			server.registerMBean( svc, objName );
			logger.debug( this, "add service:" + svc.getName()
								+ ":" + svc.getClass().getName() );
		} catch ( Exception ex ) {
		}
	}

	private int id = 0;
	/**
	* デーモンのStatusが変更された時に呼び出されます。
	* イベントの定義については{@link StatusChangeEvent}を参照してください。
	*/
	public void statusChanged( StatusChangeEvent ev ) {
		IDaemon dmn = (IDaemon )ev.getSource();
		AttributeChangeNotification an;
		an = new AttributeChangeNotification(
			dmn, id++,
			new Date().getTime(), dmn.getName()+" status changed.",
			"State", "java.lang.Integer",
			ev.getOldStatus(), ev.getNewStatus() );
		broadcaster.sendNotification( an );
	}

	/**
	* デーモンマネージャの起動処理です。
	* @param	args	コマンドラインパラメータ
	*/
	public static void main( String[] args ) {
		JMXManager obj = null;
		try {
			obj = new JMXManager();

			obj.init();
			obj.start();
			obj.mainLoop();
		} catch ( Exception ex ) {
			logger.error( JMXManager.class, "error", ex );
			ex.printStackTrace();
			System.exit( 9 );
		}
	}
}
